==> brython-3.11.gh.tar.gz: brython-3.11/www/tests/unittests/HTMLTestRunner.py <==
"""
A TestRunner for use with the Python unit testing framework. It
generates a HTML report to show the result at a glance.

The simplest way to use this is to invoke its main method. E.g.

    import unittest
    import HTMLTestRunner

    ... define your tests ...

    if __name__ == '__main__':
        HTMLTestRunner.main()


For more customization options, instantiates a HTMLTestRunner object.
HTMLTestRunner is a counterpart to unittest's TextTestRunner. E.g.

    # output to a file
    fp = file('my_report.html', 'wb')
    runner = HTMLTestRunner.HTMLTestRunner(
                stream=fp,
                title='My unit test',
                description='This demonstrates the report output by HTMLTestRunner.'
                )

    # Use an external stylesheet.
    # See the Template_mixin class for more customizable options
    runner.STYLESHEET_TMPL = '<link rel="stylesheet" href="my_stylesheet.css" type="text/css">'

    # run the test
    runner.run(my_test_suite)


------------------------------------------------------------------------
Copyright (c) 2004-2007, Wai Yip Tung
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name Wai Yip Tung nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# URL: http://tungwaiyip.info/software/HTMLTestRunner.html

__author__ = "Wai Yip Tung"
__version__ = "0.8.2"


"""
Change History

Version 0.8.2
* Show output inline instead of popup window (Viorel Lupu).

Version in 0.8.1
* Validated XHTML (Wolfgang Borgert).
* Added description of test classes and test cases.

Version in 0.8.0
* Define Template_mixin class for customization.
* Workaround a IE 6 bug that it does not treat <script> block as CDATA.

Version in 0.7.1
* Back port to Python 2.3 (Frank Horowitz).
* Fix missing scroll bars in detail log (Podi).
"""

# TODO: color stderr
# TODO: simplify javascript using more than 1 class in the class attribute?

import datetime
#import StringIO
import io as StringIO
import sys
import time
import unittest
from browser import document, html, alert

TestResult = unittest.TestResult

class _TestResult(TestResult):
    # note: _TestResult is a pure representation of results.
    # It lacks the output and reporting ability compares to unittest._TextTestResult.

    def __init__(self, verbosity=1, report_success=False,
                 report_too_long=True):
        TestResult.__init__(self)
        self.stdout0 = None
        self.stderr0 = None
        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0
        self.verbosity = verbosity

        # result is a list of result in 4 tuple
        # (
        #   result code (0: success; 1: fail; 2: error),
        #   TestCase object,
        #   Test output (byte string),
        #   stack trace,
        # )
        self.result = []
        self.report_success = report_success
        self.report_too_long = report_too_long

    def startTest(self, test):
        TestResult.startTest(self, test)
        test.startTime = time.time()
        method = getattr(test.__class__, test._testMethodName)
        test.lineno = method.__code__.co_firstlineno

    def ident(self, test):
        return [html.TD(test.__class__.__name__),
            html.TD(test._testMethodName),
            html.TD(test.lineno, Class="number"),
            html.TD(round(1000*(time.time() - test.startTime)),
                Class="number")]

    def addSubTest(self, test, subtest, err):
        TestResult.addSubTest(self, test, subtest, err)
        # special case, don't use ident()
        if not self.report_success:
            return
        cell2 = (test._testMethodName + html.BR() +
            str(subtest._subDescription())[:50])

        row = html.TR([html.TD(test.__class__.__name__),
            html.TD(cell2),
            html.TD(test.lineno, Class="number"),
            html.TD(round(1000*(time.time() - test.startTime)),
                Class="number")], Class="method")

        row <= html.TD('ok', colspan=2, Class="report_cell")
        # reset startTime
        test.startTime = time.time()
        document['report'] <= row

    def addSuccess(self, test):
        self.success_count += 1
        TestResult.addSuccess(self, test)
        ident = self.ident(test)
        row = html.TR(ident, Class="method")
        row <= html.TD('ok', colspan=2, Class="report_cell")
        if self.report_success:
            document['report'] <= row
        elif self.report_too_long and int(ident[3].text) > 3000:
            document['report'] <= row

    def addError(self, test, err):
        self.error_count += 1
        TestResult.addError(self, test, err)
        row = html.TR(self.ident(test), Class="method")
        row <= html.TD('error', Class="report_cell")
        row <= (html.TD(x) for x in self.excInfos(test, err))
        document['report'] <= row

    def addFailure(self, test, err):
        self.failure_count += 1
        TestResult.addFailure(self, test, err)
        row = html.TR(self.ident(test), Class="method")
        row <= html.TD('fail', Class="report_cell")
        row <= (html.TD(x) for x in self.excInfos(test, err))
        document['report'] <= row

    def excInfos(self, test, err):
        tb = err[2]
        infos = []
        while tb:
            fname = tb.tb_frame.f_code.co_filename
            lineno = tb.tb_frame.f_lineno
            if fname == sys.modules[test.__class__.__module__].__file__:
                infos.append(lineno)
            tb = tb.tb_next
        infos = infos or ['unknown']
        try:
            str(err[1]).splitlines()
        except:
            alert(err[1])
            alert(type(err[1]))
        lines = "\n".join(f"line {line}" for line in infos[:-1])
        return [html.TD(lines + "\nline %s - %s: %s" %(infos[-1],
            err[0].__name__,
            str(err[1]).splitlines()[0].replace('<', '&lt;')),
                Class="error_message")]

class HTMLTestRunner:
    """
    """
    def __init__(self, stream=sys.stdout, verbosity=1, title=None,
                 description=None):
        self.verbosity = 1
        self.startTime = datetime.datetime.now()

    def run(self, test, report_success=False):
        "Run the given test case or test suite."
        t = html.TABLE(Id="report", border=1)
        t <= html.TR(html.TH(x, Class="header")
            for x in ('Test class', 'Method', 'Line', 'Duration (ms)', 'Result',
            'Error message'))
        document["container"] <= t
        result = _TestResult(self.verbosity, report_success=report_success)
        test(result)
        self.stopTime = datetime.datetime.now()

    def sortResult(self, result_list):
        # unittest does not seems to run in any particular order.
        # Here at least we want to group them together by class.
        rmap = {}
        classes = []
        for n,t,o,e in result_list:
            cls = t.__class__
            if cls not in rmap:
                rmap[cls] = []
                classes.append(cls)
            rmap[cls].append((n,t,o,e))
        r = [(cls, rmap[cls]) for cls in classes]
        return r


##############################################################################
# Facilities for running tests from the command line
##############################################################################

# Note: Reuse unittest.TestProgram to launch test. In the future we may
# build our own launcher to support more specific command line
# parameters like test title, CSS, etc.
class TestProgram(unittest.TestProgram):
    """
    A variation of the unittest.TestProgram. Please refer to the base
    class for command line parameters.
    """
    def runTests(self):
        # Pick HTMLTestRunner as the default test runner.
        # base class's testRunner parameter is not useful because it means
        # we have to instantiate HTMLTestRunner before we know self.verbosity.
        if self.testRunner is None:
            self.testRunner = HTMLTestRunner(verbosity=self.verbosity)
        unittest.TestProgram.runTests(self)

main = TestProgram

##############################################################################
# Executing this module from the command line
##############################################################################

if __name__ == "__main__":
    main(module=None)

==> brython-3.11.2.gh.tar.gz: brython-3.11.2/www/tests/unittests/HTMLTestRunner.py <==
"""
A TestRunner for use with the Python unit testing framework. It
generates a HTML report to show the result at a glance.

The simplest way to use this is to invoke its main method. E.g.

    import unittest
    import HTMLTestRunner

    ... define your tests ...

    if __name__ == '__main__':
        HTMLTestRunner.main()


For more customization options, instantiates a HTMLTestRunner object.
HTMLTestRunner is a counterpart to unittest's TextTestRunner. E.g.

    # output to a file
    fp = file('my_report.html', 'wb')
    runner = HTMLTestRunner.HTMLTestRunner(
                stream=fp,
                title='My unit test',
                description='This demonstrates the report output by HTMLTestRunner.'
                )

    # Use an external stylesheet.
    # See the Template_mixin class for more customizable options
    runner.STYLESHEET_TMPL = '<link rel="stylesheet" href="my_stylesheet.css" type="text/css">'

    # run the test
    runner.run(my_test_suite)


------------------------------------------------------------------------
Copyright (c) 2004-2007, Wai Yip Tung
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name Wai Yip Tung nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# URL: http://tungwaiyip.info/software/HTMLTestRunner.html

__author__ = "Wai Yip Tung"
__version__ = "0.8.2"


"""
Change History

Version 0.8.2
* Show output inline instead of popup window (Viorel Lupu).

Version in 0.8.1
* Validated XHTML (Wolfgang Borgert).
* Added description of test classes and test cases.

Version in 0.8.0
* Define Template_mixin class for customization.
* Workaround a IE 6 bug that it does not treat <script> block as CDATA.

Version in 0.7.1
* Back port to Python 2.3 (Frank Horowitz).
* Fix missing scroll bars in detail log (Podi).
"""

# TODO: color stderr
# TODO: simplify javascript using more than 1 class in the class attribute?

import datetime
#import StringIO
import io as StringIO
import sys
import time
import unittest
from browser import document, html, alert, console

TestResult = unittest.TestResult

class _TestResult(TestResult):
    # note: _TestResult is a pure representation of results.
    # It lacks the output and reporting ability compares to unittest._TextTestResult.

    def __init__(self, verbosity=1, report_success=False,
                 report_too_long=True):
        TestResult.__init__(self)
        self.stdout0 = None
        self.stderr0 = None
        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0
        self.verbosity = verbosity

        # result is a list of result in 4 tuple
        # (
        #   result code (0: success; 1: fail; 2: error),
        #   TestCase object,
        #   Test output (byte string),
        #   stack trace,
        # )
        self.result = []
        self.report_success = report_success
        self.report_too_long = report_too_long

    def startTest(self, test):
        TestResult.startTest(self, test)
        test.startTime = time.time()
        method = getattr(test.__class__, test._testMethodName)
        test.lineno = method.__code__.co_firstlineno

    def ident(self, test):
        return [html.TD(test.__class__.__name__),
            html.TD(test._testMethodName),
            html.TD(test.lineno, Class="number"),
            html.TD(round(1000*(time.time() - test.startTime)),
                Class="number")]

    def addSubTest(self, test, subtest, err):
        TestResult.addSubTest(self, test, subtest, err)
        # special case, don't use ident()
        if not self.report_success:
            return
        cell2 = (test._testMethodName + html.BR() +
            str(subtest._subDescription())[:50])

        row = html.TR([html.TD(test.__class__.__name__),
            html.TD(cell2),
            html.TD(test.lineno, Class="number"),
            html.TD(round(1000*(time.time() - test.startTime)),
                Class="number")], Class="method")

        row <= html.TD('ok', colspan=2, Class="report_cell")
        # reset startTime
        test.startTime = time.time()
        document['report'] <= row

    def addSuccess(self, test):
        self.success_count += 1
        TestResult.addSuccess(self, test)
        ident = self.ident(test)
        row = html.TR(ident, Class="method")
        row <= html.TD('ok', colspan=2, Class="report_cell")
        if self.report_success:
            document['report'] <= row
        elif self.report_too_long and int(ident[3].text) > 3000:
            document['report'] <= row

    def addError(self, test, err):
        self.error_count += 1
        TestResult.addError(self, test, err)
        row = html.TR(self.ident(test), Class="method")
        row <= html.TD('error', Class="report_cell")
        row <= (html.TD(x) for x in self.excInfos(test, err))
        document['report'] <= row

    def addFailure(self, test, err):
        self.failure_count += 1
        TestResult.addFailure(self, test, err)
        row = html.TR(self.ident(test), Class="method")
        row <= html.TD('fail', Class="report_cell")
        row <= (html.TD(x) for x in self.excInfos(test, err))
        document['report'] <= row

    def excInfos(self, test, err):
        tb = err[2]
        try:
            tb.pos = 0
        except:
            pass
        infos = []
        while tb:
            fname = tb.tb_frame.f_code.co_filename
            lineno = tb.tb_frame.f_lineno
            if fname == sys.modules[test.__class__.__module__].__file__:
                infos.append(lineno)
            tb = tb.tb_next
        if not infos:
            # Brython-specific hack
            try:
                infos.append(err[2].linenos[0])
            except:
                pass
        infos = infos or ['unknown']
        try:
            str(err[1]).splitlines()
        except:
            alert(err[1])
            alert(type(err[1]))
        lines = "\n".join(f"line {line}" for line in infos[:-1])
        console.log('err', err)
        message = str(err[1])
        if message:
            message = message.splitlines()[0].replace('<', '&lt;')
        return [html.TD(lines + "\nline %s - %s: %s" %(infos[-1],
            err[0].__name__,
            message),
            Class="error_message")]

class HTMLTestRunner:
    """
    """
    def __init__(self, stream=sys.stdout, verbosity=1, title=None,
                 description=None):
        self.verbosity = 1
        self.startTime = datetime.datetime.now()

    def run(self, test, report_success=False):
        "Run the given test case or test suite."
        t = html.TABLE(Id="report", border=1)
        t <= html.TR(html.TH(x, Class="header")
            for x in ('Test class', 'Method', 'Line', 'Duration (ms)', 'Result',
            'Error message'))
        document["container"] <= t
        result = _TestResult(self.verbosity, report_success=report_success)
        test(result)
        self.stopTime = datetime.datetime.now()

    def sortResult(self, result_list):
        # unittest does not seems to run in any particular order.
        # Here at least we want to group them together by class.
        rmap = {}
        classes = []
        for n,t,o,e in result_list:
            cls = t.__class__
            if cls not in rmap:
                rmap[cls] = []
                classes.append(cls)
            rmap[cls].append((n,t,o,e))
        r = [(cls, rmap[cls]) for cls in classes]
        return r


##############################################################################
# Facilities for running tests from the command line
##############################################################################

# Note: Reuse unittest.TestProgram to launch test. In the future we may
# build our own launcher to support more specific command line
# parameters like test title, CSS, etc.
class TestProgram(unittest.TestProgram):
    """
    A variation of the unittest.TestProgram. Please refer to the base
    class for command line parameters.
    """
    def runTests(self):
        # Pick HTMLTestRunner as the default test runner.
        # base class's testRunner parameter is not useful because it means
        # we have to instantiate HTMLTestRunner before we know self.verbosity.
        if self.testRunner is None:
            self.testRunner = HTMLTestRunner(verbosity=self.verbosity)
        unittest.TestProgram.runTests(self)

main = TestProgram

##############################################################################
# Executing this module from the command line
##############################################################################

if __name__ == "__main__":
    main(module=None)
